package org.cocos2dx.javascript.util;

import android.content.Context;
import android.content.pm.ApplicationInfo;
import android.content.pm.PackageInfo;

import org.cocos2dx.javascript.bean.AppInfo;

import java.util.ArrayList;
import java.util.List;

public class PackageUtil {
    private ArrayList<AppInfo> mSystemAppList;
    private ArrayList<AppInfo> mOtherAppList;
    private ImageUtil mImageUtil;

    public PackageUtil(Context context) {
        mImageUtil = new ImageUtil();
        getPackageAll(context);
    }


    public List<AppInfo> getSystemAppList() {
        return mSystemAppList;
    }

    public List<AppInfo> getOtherAppList() {
        return mOtherAppList;
    }


    public void getPackageAll(final Context context) {
        new Thread(new Runnable() {
            public void run() {
                // a potentially time consuming task
                mSystemAppList = new ArrayList<>();
                mOtherAppList = new ArrayList<>();
                try {
                    List<PackageInfo> packageInfos = context.getPackageManager().getInstalledPackages(context.getPackageManager().GET_ACTIVITIES |
                            context.getPackageManager().GET_SERVICES);
                    for (PackageInfo packageInfo : packageInfos) {
                        AppInfo tmpInfo = new AppInfo();
                        tmpInfo.appName = packageInfo.applicationInfo.loadLabel(context.getPackageManager()).toString();
                        tmpInfo.packageName = packageInfo.packageName;
                        tmpInfo.versionName = packageInfo.versionName;
                        tmpInfo.versionCode = packageInfo.versionCode;
                        tmpInfo.appIcon = "data:image/jpg;base64," + mImageUtil.bitmapToBase64(mImageUtil.DrawableToBitmap(packageInfo.applicationInfo.loadIcon(context.getPackageManager())));
                        if (isSystemApp(packageInfo)) {
                            tmpInfo.isSystemApp = 0;
                            mSystemAppList.add(tmpInfo);
                        } else {
                            tmpInfo.isSystemApp = 1;
                            mOtherAppList.add(tmpInfo);
                        }
                    }
                } catch (Throwable t) {
                    t.printStackTrace();
                }
            }
        }).start();

    }

    // 通过packName得到PackageInfo，作为参数传入即可
    private boolean isSystemApp(PackageInfo pi) {
        boolean isSysApp = (pi.applicationInfo.flags & ApplicationInfo.FLAG_SYSTEM) == 1;
        boolean isSysUpd = (pi.applicationInfo.flags & ApplicationInfo.FLAG_UPDATED_SYSTEM_APP) == 1;
        return isSysApp || isSysUpd;
    }
}
